/*
 * linux/sound/arm/ep93xx-alsa-ac97.c -- ALSA PCM interface for the edb93xx ac97 audio
 *
 * Author:      Shrek Wu
 * Created:     July 19, 2005
 * Copyright:   Cirrus Logic, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
 
#define AC97_SLOT_0 	0x0001
#define AC97_SLOT_1 	0x0002
#define AC97_SLOT_2 	0x0004
#define AC97_SLOT_3 	0x0008
#define AC97_SLOT_4 	0x0010
#define AC97_SLOT_5 	0x0020
#define AC97_SLOT_6 	0x0040
#define AC97_SLOT_7 	0x0080
#define AC97_SLOT_8 	0x0100
#define AC97_SLOT_9 	0x0200
#define AC97_SLOT_10 	0x0400
#define AC97_SLOT_11 	0x0800
#define AC97_SLOT_12 	0x1000



#define I2S_SCALE_BUFFERS

/*
 *  Getting this from the rme96xx driver.  Has same numerical value
 *  as AC3.  Hmmmm.
 */
#ifndef AFMT_S32_BLOCKED
#define AFMT_S32_BLOCKED 0x0000400
#endif

#define EP93XX_DEFAULT_NUM_CHANNELS      2
#define EP93XX_DEFAULT_FORMAT            AFMT_S16_LE
#define EP93XX_DEFAULT_BIT_WIDTH         16

/*
 * Buffer Management
 */

typedef struct {

	int 				size;			/* buffer size 						*/
	int 				reportedsize;			/* buffer size 						*/
	int 				sent;			/* indicates that dma has the buf	*/
	char 				*start;			/* points to actual buffer 			*/
	dma_addr_t 			dma_addr;		/* physical buffer address 			*/
	struct semaphore 	sem;			/* down before touching the buffer 	*/
	int 				master;			/* owner for buffer allocation,    	*/
										/* contains size when true 			*/
	struct audio_stream_s 	*stream;	/* owning stream 					*/

} audio_buf_t;

#define MAX_DEVICE_NAME 20

typedef struct audio_stream_s {

	/* dma stuff */
	int				dmahandles[3];		/* handles for dma driver instances */
	char			devicename[MAX_DEVICE_NAME]; /* string - name of device */
	int				NumDmaChannels;		/* 1, 2, or 3 DMA channels 			*/
	
	audio_buf_t 	*buffers;			/* array of audio buffer structures */
	audio_buf_t 	*buf;				/* current buffer used by read/write */
	u_int 			buf_idx;			/* index for the pointer above... 	*/
	u_int 			fragsize;			/* fragment i.e. buffer size 		*/
	u_int 			nbfrags;			/* nbr of fragments i.e. buffers 	*/
	u_int			reportedfrags;		/* # of frags reported to app		*/
	u_int 			externedfragsize;			/* fragment i.e. buffer size 		*/
	int 			bytecount;			/* nbr of processed bytes 			*/
	int 			externedbytecount;			/* nbr of processed bytes 			*/
	int 			getptrCount;		/* value of bytecount last time 	*/
										/* anyone asked via GETxPTR 		*/
	int 			fragcount;			/* nbr of fragment transitions 		*/
	ep93xx_dma_int_t dma_ch;			/* DMA channel ID 					*/
	wait_queue_head_t 	wq;				/* for poll 						*/
	int 			mapped;			/* mmap()'ed buffers 				*/
	int 			active;			/* actually in progress 			*/
	int 			stopped;			/* might be active but stopped 		*/

	/*
	 * Data about the file format that we are configured to play.
	 */
	long 				audio_num_channels;			/* Range: 1 to 6 */
	long 				audio_format;
	long 				audio_stream_bitwidth;		/* Range: 8, 16, 24, 32 */

} audio_stream_t;


/*
 * State structure for one instance
 */
typedef struct {
	audio_stream_t 		*output_stream;
	audio_stream_t 		*input_stream;
	
	ep93xx_dma_dev_t 	output_dma;
	ep93xx_dma_dev_t 	input_dma;
	
	char 				*output_id;
	char 				*input_id;
	
	int 				rd_ref:1;			/* open reference for recording */
	int 				wr_ref:1;			/* open reference for playback */
	
	/*
	 * Pointers to init, shutdown, and ioctl functions.
	 */
	void 				(*hw_enable)(int input_or_output_stream);
	void 				(*hw_disable)(int input_or_output_stream);
	int 				(*client_ioctl)(struct inode *, struct file *, uint, ulong);
	void				(*set_hw_serial_format)( long );

	/*
	 *
	 */
	struct 				semaphore Loopsem;		/* to protect against races in attach() */
	int (*startup)(snd_pcm_substream_t *);
	void (*shutdown)(snd_pcm_substream_t *);
	int (*prepare)(snd_pcm_substream_t *);

	/*
	 *
	 */

	struct 				semaphore sem;		/* to protect against races in attach() */

	/*
	 * Data about how the audio controller and codec are set up.
	 */
	int					DAC_bit_width;		/* 16, 20, 24 bits */
	int					bCompactMode;		/* set if 32bits = a stereo sample */

} audio_state_t;






extern int ep93xx_ac97_pcm_new(snd_card_t *card, audio_state_t *state, snd_pcm_t **rpcm);

